<?php

/**
 * Returns formatted microtime
 * @return long
 */
function wscGetFormattedMicrotime()
{
    list($usec, $sec) = explode(' ', microtime());
    return ((float)$usec + (float)$sec);    
}

/**
 * Recursively checks the destination directory
 * @param string $dir
 * @param array &$settings
 * @param string $task
 * @param string $mode
 * @return bool
 */
function wscCheckRecursive($dir, &$settings, $task = 'check', $mode = 'view') 
{ 
    if(!$dh = @opendir($dir)) return false;
    
    // sets time difference in seconds
    $diff = 60 * 60 * (int)$settings['changed']['diff_hours'];
    // prepare allowed infected extensions
    $allowedExtensions = isset($settings['infected']['allowed_extensions']) && is_array($settings['infected']['allowed_extensions']) ? $settings['infected']['allowed_extensions'] : array();
    $allowedExtensions = implode('|', $allowedExtensions);
    $infectedExtensions = str_ireplace(array('.'), '\.', $allowedExtensions);
    // prepare allowed search string
    $searchString = str_ireplace(array('(', ')'), array('\(', '\)'), $settings['infected']['search_string']);

    while(false !== ($obj = readdir($dh))){
        if($obj == '.' || $obj == '..') continue;
        
        $path = $dir.$obj;
        
        if($settings['general']['check_subdirectories'] && is_dir($path.'/')){
            $settings['general']['directories_checked']++;
            wscCheckRecursive($path.'/', $settings, $task, $mode);            
        }
        
        if($obj != '.' && $obj != '..'){            

            $settings['general']['files_checked']++;            

            // check for thuumbs            
            if($task == 'check' && $settings['thumb']['active'] && strtolower($obj) == 'thumbs.db'){
                $settings['thumb']['found_files'][] = $path;
                $settings['thumb']['total_found']++;
            }
            
            // check for cache files
            if($task == 'check' && $settings['cache']['active'] && preg_match('/\.'.trim($settings['cache']['file_extension'], '.').'/i', $obj)){
                $settings['cache']['found_files'][] = $path;
                $settings['cache']['total_found']++;
            }
            
            // check for infected files
            if($task == 'check' && $settings['infected']['active'] && preg_match('/'.$infectedExtensions.'/i', $obj)){
                $content = @file_get_contents($path);
                if($content !== false && preg_match("/$searchString/", $content)){
                    $settings['infected']['found_files'][] = $path;
                    $settings['infected']['total_found']++;
                }                
            }

            // check for last changed files
            if($task == 'check' && $settings['changed']['active'] && (time() - @filemtime($path)) < $diff){
                if(!is_dir($path)){
                    $skip = false;
                    if($settings['changed']['ignore_cache_files'] && preg_match('/\.'.trim($settings['cache']['file_extension'], '.').'/i', $obj)){
                        $skip = true;
                    }

                    if(!$skip){
                        $settings['changed']['found_files'][] = array('name'=>$path, 'date_changed'=>@filemtime($path));
                        $settings['changed']['total_found']++;
                    }
                }
            }

            // check for error log files
            if($task == 'check' && $settings['errorlog']['active'] && $settings['errorlog']['file_name'] == $obj){
                $settings['errorlog']['found_files'][] = $path;
                $settings['errorlog']['total_found']++;
            }

        }
    } 
    closedir($dh);     
    return true; 
}

/**
 * Returns shorten name of the given file
 * @param string $file
 * @param int $lengthFirst
 * @param int $lengthLast
 * @return string
 */
function wscCreateShortenName($file, $lengthFirst = 50, $lengthLast = 20)
{
    return preg_replace("/(?<=.{{$lengthFirst}})(.+)(?=.{{$lengthLast}})/", "...", $file);  
}

/**
 * Draws search result on the screen
 * @param array &$settings
 * @param string $type
 * @param bool $deleteLink
 * @param bool $cleanLink
 * @return text/html
 */
function wscDrawResult(&$settings, $type = '', $deleteLink = false, $cleanLink = false)
{
    $token = isset($_SESSION['token']) ? $_SESSION['token'] : '';
    $output = '<p class="text-success">';
    $output .= 'Check is done!<br>';
    $output .= 'Found: <span id="counted_'.$type.'">'. $settings[$type]['total_found'].'</span> files';        
    if($settings[$type]['total_found'] > 0){
        $output .= ' <span class="separator">/</span> <a id="'.$type.'_files_conteiner_link_show" class="files-container-link" href="javascript:void(0)" onclick="wscContainerToggle(\''.$type.'_files_conteiner\',\'show\')">Show<i class="icon-folder-close"></i></a>';
        $output .= ' <a id="'.$type.'_files_conteiner_link_hide" class="files-container-link" style="display:none;" href="javascript:void(0)" onclick="wscContainerToggle(\''.$type.'_files_conteiner\',\'hide\')">Hide<i class="icon-folder-open"></i></a>';
        $output .= '<div id="'.$type.'_files_conteiner" class="files-container" style="display:none;">';
        if(is_array($settings[$type]['found_files'])){
            $count = 0;
            foreach($settings[$type]['found_files'] as $file){
                $id = $type.'_file_'.++$count;
                $output .= '<div id="'.$id.'">';
                if($deleteLink){
                    $output .= '<a href="javascript:void(0)" onclick="javascript:wscDeleteFile(\''.$id.'\',\''.$file.'\',\''.$type.'\',\''.$token.'\')">';
                    $output .= '<i class="icon-trash"></i>Delete</a> ';
                }
                if($cleanLink){
                    $output .= '<a href="javascript:void(0)" onclick="javascript:wscCleanFile(\''.$id.'\',\''.$file.'\',\''.$type.'\',\''.$token.'\')">';
                    $output .= '<i class="icon-wrench"></i>Clean</a> ';                    
                }

                if(is_array($file)){
                    $shortenName = wscCreateShortenName($file['name']);
                    $title = (!is_array($file) && $file['name'] !== $shortenName) ? htmlentities($file) : '';                    
                }else{
                    $shortenName = wscCreateShortenName($file);
                    $title = ($file !== $shortenName) ? htmlentities($file) : '';
                }

                if($type == 'changed'){
                    $output .= @date('Y-m-d H:i:s', $file['date_changed']).' <span title="'.$title.'">'.$shortenName.'</span>';
                }else{
                    $output .= '<span title="'.$title.'">'.$shortenName.'</span>';
                }

                $output .= "</div>";
                $output .= "\r\n";
            }                                    
        }
        $output .= '</div>';        
    }
    $output .= '</p>';
    
    return $output;    
}
